@@ -5,7 +5,7 @@ use POSIX qw(strftime);
 use File::Spec;
 use File::MimeInfo;
 
-our $VERSION = 0.02;
+our $VERSION = 0.03;
 
 =head1 NAME
 
@@ -67,96 +67,29 @@ sub new {
 	return $self;
 }
 
-=cut
-
-# proof of concept plugin code
-
-sub _scan_for_plugins {
-	my ($self, $dir) = @_;
-	return unless -d $dir;
-	#print "scanning $dir\n";
-	my $file = File::Spec->catfile($dir, '.zim.template');
-	if (-f $file) {
-		print "found $file\n";
-		$file = _catfile($file);
-		if ($file =~ /^module=(.+)$/m) {
-			my $class = $1;
-			eval "use $class";
-			die if $@;
-			my $page = $self->get_pagename($dir);
-			$self->{plugins}{$page} = $class->new($dir);
-			print "found plugin for namespace $page";
-			return;
-		}
-	}
-	opendir DIR, $dir or die "Could not list $dir\n";
-	my @items = grep {! /^\./} sort readdir DIR;
-	close DIR;
-	for (@items) {
-		my $subdir = File::Spec->catdir($dir, $_);
-		$self->_scan_for_plugins($subdir) if -d $subdir;
-	}
-}
-
-sub belongs_to_plugin {
-	my ($self, $page) = @_;
-	my ($plug) = grep {$page =~ /^$_:/} keys %{$self->{plugins}};
-	return $plug;
-}
-
-sub get_pagename {
-	my ($self, $file) = @_;
-	$file = File::Spec->canonpath($file);
-	$file =~ s/^$self->{root}//
-		or die "file $file outside root $self->{root}\n";
-	$file =~ s/\.txt//i;
-	my @parts = File::Spec->splitdir($file);
-	my $page = join ':', @parts;
-	$page =~ s/^://;
-	return $page;
-}
-
-=cut
-
 =item list_pages(NAMESPACE)
 
-Without NAMESPACE list all pages in the root directory.
+Lists pages in NAMESPACE. Sub-namespaces have a trailing ':'
+in the listing.
 
 =cut
 
 sub list_pages {
-	my ($self, $namespace, $callback) = @_;
-	my $dir = File::Spec->catdir($self->{root}, split /:+/, $namespace);
-	return unless -d $dir;
-	unless ($callback) {
-		my @pages;
-		$callback = sub {push @pages, @_};
-		_list_pages($dir, $namespace, $callback);
-		return @pages;
-	}
-	else {  _list_pages($dir, $namespace, $callback)  }
-}
-
-sub _list_pages {
-	my ($dir, $pref, $callback) = @_;
-	my @pages;
-	opendir DIR, $dir or die "Could not list $dir\n";
-	my @items = grep {! /^\./} sort readdir DIR;
-	closedir DIR;
-	for (@items) {
-		my $child = File::Spec->catdir($dir, $_);
-		my $name  = length($pref) ? "$pref:$_" : $_;
-		if (-d $child) {
-			$name =~ s/_files$//;
-			_list_pages($child, $name, $callback); # recurs
-		}
+	my ($self, $namespace) = @_;
+	my $dir = $self->filename($namespace, 'DIR');
+	return () unless -d $dir;
+	opendir DIR, $dir or die "Could not list dir $dir\n";
+	my @items = grep {defined $_} map {
+		my $item = File::Spec->catfile($dir, $_);
+		if (-d $item) { s/(_files)?$/:/ }
 		else {
-			next unless mimetype($child) =~ /^text/;
-			$name =~ s/\.txt//i;
-			push @pages, $name;
+			if (mimetype($item) =~ /^text/) { s/\.txt$// }
+			else { $_ = undef }
 		}
-	}
-	$callback->(@pages);
+		$_;
+	} grep {! /^\./} readdir DIR;
+	closedir DIR;
+	return sort {lc($a) cmp lc($b)} @items;
 }
 
 =item page_exists(PAGE_NAME)
@@ -193,7 +126,10 @@ sub load_page {
 	my $page = Zim::Page::File->new($self, $name);
 	#print "page $page => file ".$page->filename."\n";
 	if (-f $page->filename) { $page->read_file }
-	else { $page->push_blocks( $self->template($name) ) }
+	else {
+		$page->push_blocks( $self->_template($name) );
+		$page->{status} = 'new';
+	}
 
 	return $page;
 }
@@ -209,9 +145,12 @@ sub save_page {
 
 	if ($page->isa('Zim::Page::File')) {
 		$page->write_file;
+		$page->{status} = '';
+		return 1;
 	}
 	else {
 		warn "Can't save page ".$page->name."\n";
+		return 0;
 	}
 }
 
@@ -256,19 +195,14 @@ sub delete_page {
 	if ($page->isa('Zim::Page::File')) {
 		$page->remove_file
 			if -f $page->filename;
+		$page->{status} = 'deleted';
 	}
 	else {
 		warn "Can't delete page ".$page->pagename."\n";
 	}
 }
 
-=item template(NAME)
-
-Returns contents for a new page as blocks. 
-
-=cut
-
-sub template {
+sub _template {
 	# FIXME make template configurable
 	my ($self, $name) = @_;
 	$name =~ /([^:]+):*$/;
@@ -289,20 +223,23 @@ and should therefor never be called directly from the GUI.
 
 =over 4
 
-=item filename(NAME)
+=item filename(NAME, DIR)
 
 Gives the filename corresponding to a page name.
 NAME is a string. (This lookup is more difficult then it might seem.)
 
-In list context filname, directory and real page name are returned.
+DIR is a boolean that can be set to tell that NAME is expected to
+be a directory.
+
+In list context filename, directory and real page name are returned.
 
 =cut
 
 sub filename {
-	my ($self, $page) = @_;
+	my ($self, $page, $is_dir) = @_;
 
 	my @dirs = split /:+/, $page;
-	my $basename = pop @dirs;
+	my $basename = pop @dirs unless $is_dir;
 
 	# check the dirs
 	my @realname;
@@ -323,6 +260,9 @@ sub filename {
 		$dir = File::Spec->catdir($self->{root}, @dirs);
 	}
 
+	return wantarray ? ($dir, $dir, join ':', @realname) : $dir
+		if $is_dir;
+
 	# check which filename to use
 	my $file;
 	for ($basename, lc($basename)) {
@@ -346,11 +286,11 @@ need to exist and can be a directory as well as a file.
 =cut
 
 sub pagename {
-	my $file = shift;
-	my ($vol, $dirs, $page) = File::Spec->splitpath($file);
-	my @parts = map s/_files$//, File::Spec->splitdir($dirs);
-	$page =~ s/\.txt$//;
-	return join ':', @parts, $page;
+	my ($self, $file) = @_;
+	# $file =~ s/^\Q$self->{root}\E//;
+	my @parts = map  {s/_files$//; $_} File::Spec->splitdir($file);
+	$parts[-1] =~ s/\.txt$//;
+	return join ':', @parts;
 }
 
 package Zim::Formatter;
@@ -375,6 +315,9 @@ sub text2blocks {
 	# for headers it depends on the source whether they are
 	# followed by a empty line or not, for paragraphs (normal,
 	# pre, lists, etc.) we always append an extra line break
+	#
+	# FIXME because undef is returned for empty lines blocks of
+	# empty lines disappear
 	my ($self, @text) = @_;
 	return grep {defined $_} map {
 		(! /\S/)        ?  undef                         :
@@ -401,7 +344,7 @@ sub parse_pre { # parse pre formated paragraphs
 	return ['pre' => $pre];
 }
 
-our @parser_subs = qw/parse_link parse_styles/;
+our @parser_subs = qw/parse_image parse_link parse_styles/;
 
 sub parse_block { # parse a block of text
 	my ($self, @text) = @_;
@@ -411,6 +354,14 @@ sub parse_block { # parse a block of text
 	return @text;
 }
 
+sub parse_image {
+	my ($self, $text) = @_;
+	my $i = 0;
+	return map {
+		($i++ % 2) ? ['image', $_] : $_
+	} split /\{\{(.+?)\}\}/, $text;
+}
+
 sub parse_link { # parse links and urls
 	my ($self, $text) = @_;
 	my $i = 0;
@@ -510,7 +461,7 @@ sub new {
 	}, $class ;
 }
 
-=item name()
+=item name( )
 
 Returns the name of the page.
 
@@ -518,20 +469,18 @@ Returns the name of the page.
 
 sub name { $_[0]->{pagename} }
 
-=item realname()
+=item realname( )
 
 Returns a name for the current page that can be used for comparing
 page objects. By default the real name is the same as the page name,
 but for example for file based pages where a case insensitive lookup
 is done these will differ.
 
-If the page doesn't really exist undef is returned.
-
 =cut
 
 sub realname { $_[0]->{pagename} }
 
-=item basename()
+=item basename( )
 
 Returns the last part of the page name.
 
@@ -543,9 +492,10 @@ sub basename {
 	return $name;
 }
 
-=item namespace()
+=item namespace( )
 
 Returns the first part of the page name.
+Including a trailing ':'.
 
 =cut
 
@@ -555,6 +505,19 @@ sub namespace {
 	return defined($1) ? $1 : ':';
 }
 
+=item status(STRING)
+
+Set or get a status string for this page.
+Typical status strings are 'new' and 'deleted'.
+
+=cut
+
+sub status {
+	my ($self, $status) = @_;
+	$self->{status} = $status if defined $status;
+	return $self->{status};
+}
+
 =item resolv_link(LINK)
 
 Returns a page name for a page linked from this page.
@@ -564,13 +527,13 @@ This method allows you to have relative links.
 
 sub resolve_link {
 	my ($self, $link) = @_;
-	unless ($link =~ /:/) { # relative link
-		$link = $self->namespace.":$link";
-	}
+	$link = ($link =~ s/^\.//) ? $self->name.':'.$link : # sub namespace
+		($link !~ /:/) ? $link = $self->namespace.$link : # relative link
+		$link ;
 	return $link;
 }
 
-=item read_block()
+=item read_block( )
 
 Returns one block of data or undef on the end of the list.
 Always use this method to access the data in a page.
@@ -586,9 +549,9 @@ sub read_block {
 	return $self->{data}[ $self->{iter}++ ];
 }
 
-=item rewind()
+=item rewind( )
 
-Resets the iterator for L<read_block()>.
+Resets the iterator for L<read_block( )>.
 
 =cut
 
@@ -637,7 +600,7 @@ sub unshift_block {
 	}
 }
 
-=item clean()
+=item clean( )
 
 Logical cleanup of the data tree.
 
@@ -645,7 +608,7 @@ Logical cleanup of the data tree.
 
 sub clean {
 	my $self = shift;
-	for my $i (0 .. $#{$self->{data}}) {
+	for my $i (1 .. $#{$self->{data}}) {
 		if (ref $self->{data}[$i]) {
 			if ($self->{data}[$i][0] eq 'normal') {
 				my (undef, @text) = @{$self->{data}[$i]};
@@ -661,7 +624,7 @@ sub clean {
 	@{$self->{data}} = grep defined($_), @{$self->{data}};
 }
 
-=item clear()
+=item clear( )
 
 Clear all data from this page.
 
@@ -672,11 +635,11 @@ sub clear {
 	$_[0]->{iter} = 0;
 }
 
-=item save()
+=item save( )
 
-=item delete()
+=item delete( )
 
-=item move()
+=item move( )
 
 The methods C<save()>, C<delete()> and C<move()> are aliases
 for the methods C<save_page()>, C<delete_page()> and C<move_page()>
@@ -690,71 +653,6 @@ sub delete { $_[0]->{zim}->delete_page(@_) }
 
 sub move { $_[0]->{zim}->move_page(@_) }
 
-package Zim::Page::Untitled;
-
-=head2 Zim::Page::Untitled
-
-This class provides for a page that doesn't have a name yet.
-It can be used to avoid pages names like 'untitled1', 'untitles2', etc.
-
-When you are done editing it you can let it generate a name based
-on the content.
-
-This class derives from Zim::Page.
-
-=over 4
-
-=item new(PARENT, NAMESPACE)
-
-Although an untitled page doesn't have a name yet it can allready 
-have a namespace.
-
-=cut
-
-our @ISA = qw/Zim::Page/;
-
-sub new {
-	my ($class, $parent, $name) = @_;
-	$name =~ s/^:+|:+$//g;
-	bless {
-		zim => $parent,
-		namespace => $name,
-		iter => 0,
-		data => [],
-	}, $class ;
-}
-
-sub name { return $_[0]->{namespace} . ':__Untitled__' }
-
-sub realname { return undef }
-
-sub basename { return '__Untitled__' }
-
-sub namespace { return $_[0]->{namespace} }
-
-=item findname()
-
-Search the content for a nice name. This will be either
-the first heading or the first line of the content.
-
-=cut
-
-sub findname {
-	my $self = shift;
-	my ($name) = map { $$_[1] }
-		grep {ref($_) and $$_[0] =~ /^head/} @{$self->{data}};
-	unless (length $name) {
-		for (@{$self->{data}}) {
-			$name = ref($_) ? $$_[0] : $_;
-			last if length $name;
-		}
-	}
-	$name =~ s/\n.*//s; # only keep first line
-	$name =~ s/\s/_/g;
-	$name =~ s/[^\w:\.\-]//g;
-	return length($name) ? $self->resolve_link($name) : undef ;
-}
-
 package Zim::Page::File;
 
 =back
@@ -793,7 +691,7 @@ sub new {
 
 sub realname { return $_[0]->{realname} }
 
-=item filename()
+=item filename( )
 
 Returns the filename for the current object.
 
@@ -805,7 +703,7 @@ sub filename {
 		: $_[0]->{filename} ;
 }
 
-=item read_file()
+=item read_file( )
 
 Reads content from the file corresponding to this page and calls
 Zim::Formatter to parse the text.
@@ -820,7 +718,7 @@ sub read_file {
 	];
 }
 
-=item write_file()
+=item write_file( )
 
 Calls Zim::Formatter to convert the data structure to plain text and
 writes this to the corresponding text file.
@@ -835,7 +733,7 @@ sub write_file {
 	);
 }
 
-=item remove_file()
+=item remove_file( )
 
 Removes the current file (effectively deleting the page) and
 also the current directory if it is empty.
@@ -984,6 +882,6 @@ modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<zim(1)>
+L<zim>(1)
 
 =cut
